@@ -9,12 +9,26 @@ chr1. The same concept is applied for all reads against all features and a
 distribution of relative positions is constructed.
 """
 
+
 import pysam
 import argparse
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.backends.backend_pdf import PdfPages
 
+
+class Feature:
+    def __init__(self, chrom, start, end, strand):
+        self.chrom = chrom
+        self.start = start
+        self.end = end
+        self.strand = strand
+
+    def __str__(self):
+        return "{}:{}-{}-{}".format(
+                self.chrom, self.start, self.end, self.strand)
+
+
 def find_pos(start, end, strand, pos):
     if pos != '5p' and pos != '3p':
         raise ValueError('Incorrectly specified position')
@@ -25,27 +39,26 @@ def find_pos(start, end, strand, pos):
         final_pos = end
     return final_pos
 
-def relative_pos(bed, read, fpos, rpos):
+
+def relative_pos(feat, read, fpos, rpos):
     """
     rel_pos calculates the relative position between the two reference
     positions of a bed and a reads entry.
     """
-    # NOTE: the relative position is supposed to indicate if one is upstream
-    # or downstream of the other. Pay attention to the strand.
-    fstart, fend, fstrand = bed[0], bed[1], bed[2]
-    feat_pos = find_pos(fstart, fend, fstrand, fpos)
+    feat_pos = find_pos(feat.start, feat.end, feat.strand, fpos)
     read_strand = '+' if read.is_forward else '-'
     read_pos = find_pos(read.reference_start, read.reference_end - 1, read_strand, rpos)
     rel_pos = read_pos - feat_pos
-    if fstrand == '-':
+    if feat.strand == '-':
         rel_pos *= -1
     return rel_pos
 
+
 def main():
-    parser = argparse.ArgumentParser()
-    parser.add_argument("-m","--bam",
+    parser = argparse.ArgumentParser(description=__doc__)
+    parser.add_argument("-m","--bam", required=True,
                     help = "BAM file with reads. Must be indexed.")
-    parser.add_argument("-b","--bed",
+    parser.add_argument("-b","--bed", required=True,
                     help = "BED file with features.")
     parser.add_argument("-u","--up", type = int, default = 100,
                     help = "Number of nts to plot upstream of pos. (default: %(default)s)")
@@ -55,56 +68,47 @@ def main():
                     help = "Reference point for features; one of 5p or 3p (default: %(default)s)")
     parser.add_argument("-r","--rpos", default='5p',
                     help = "Reference point for reads; one of 5p or 3p (default: %(default)s)")
-    parser.add_argument("-o", "--pdf",
+    parser.add_argument("-o", "--pdf", required=True,
                     help = "Output pdf file with plot")
     args = parser.parse_args()
 
-
     # Initialize a histogram
-    hist = {i : 0 for i in range(-1 * args.up, args.down + 1)}
+    positions = list(range(-1 * args.up, args.down + 1))
+    hist = {i : 0 for i in positions}
 
     # Open the bam file
     bamfile = pysam.AlignmentFile(args.bam, "rb")
 
-
-    # Open and loop on the BED file line by line. For each line query the BAM file
-    # to get overlapping reads. Calculate relative positions and add in the
-    # histogram. NOTE: BED files are 0 based files.
+    # Loop on the BED file and query the BAM file to get overlapping reads for
+    # each line. Calculate relative positions and add in the histogram.
     feature_count = 0
-    total_counts = 0
-    allowed_counts = 0
     with open(args.bed) as bed:
         for line in bed:
             feature_count += 1
             cols = line.strip().split('\t')
-            chr = cols[0]
-            start = int(cols[1])
-            end = int(cols[2]) - 1 # BED files are zero based i.e. [coord,coord)
-            strand = str(cols[5])  # Can be + or -
-            feat = [start, end, strand]
-            # Fetch overlapping BAM entries
-            reads = bamfile.fetch(chr, start, end)
-            #reads_counts.append(bamfile.count(chr, start, end))
-            # Calculate relative positions and append to histogram
+            feat = Feature(cols[0], int(cols[1]), int(cols[2]), cols[5])
+
+            if bamfile.header.get_tid(feat.chrom) == -1:
+                continue
+
+            reads = bamfile.fetch(feat.chrom, feat.start, feat.end)
             for read in reads:
-                total_counts += 1
                 rel_pos = relative_pos(feat, read, args.fpos, args.rpos)
-                if rel_pos > -1 * args.up and rel_pos < args.down:
-                    allowed_counts += 1
+                if rel_pos >= -1 * args.up and rel_pos <= args.down:
                     hist[rel_pos] += 1
 
-    positions = [int(x) for x in hist.keys()]
-    positions.sort()
-    values = [hist[i] for i in positions]
-    # print('Total features: ' + str(feature_count))
-    # print('Total reads overlap: ' + str(sum(total_counts)))
-    # print('Total reads allowed: ' + str(allowed_counts))
+    # Print a table with the histogram to stdout.
+    print("\t".join(["pos", "count"]))
+    for i in positions:
+        print("\t".join([str(i), str(hist[i])]))
 
+    # Plot the histogram in a pdf.
+    values = [hist[i] for i in positions]
     with PdfPages(args.pdf) as pages:
         fig, ax = plt.subplots(layout='constrained')
         plt.bar(positions, values)
-        plt.xlabel('Relative position to features ' + args.fpos + ' end')
-        plt.ylabel('Read count')
+        plt.xlabel('Relative position to feature ' + args.fpos + ' end')
+        plt.ylabel('Read-feature pairs count')
         pages.savefig(fig)
         plt.close()
 
